import pygame, sys
from pygame.locals import *

BLANK = 0
BLOCK = 1
FOOD = 2

MAP_WIDTH = 20
MAP_HEIGHT = 15

# 迷路のマップデータ。（まだ作っていない…。）
MAP = [
    [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],

    [1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],

    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],
    [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]
]

CHIPSIZE = 32

MAPWIDTH = 20
MAPHEIGHT = 15


class Sprite(pygame.sprite.Sprite):
    def __init__(self, filename, x, y,framex=None,framey=None):
        pygame.sprite.Sprite.__init__(self)
        self.image = image_load(filename)
        # 画像を分割するかどうかをチェックする。
        if framex and framey is None:
            width = self.image.width
            height = self.image.height
        else:
            self.imageList = split_image_load(self.image,framex,framey)
            width = self.imageList[0].get_width()
            height = self.imageList[0].get_height()

        transColor = self.image.get_at((0, 0))
        self.image.set_colorkey(transColor, RLEACCEL)
        self.rect = Rect(x, y, width, height)

        # 浮動小数点に変換
        self.fpx = float(self.rect.x)
        self.fpy = float(self.rect.y)
        # 移動量
        self.fpvx = 0.0
        self.fpvy = 0.0

class Player(Sprite):
    def __init__(self):
        Sprite.__init__(self, "PacMan.bmp", 304, 258,32,32)

    def change_speed(self, x, y):
        self.fpvx = x
        self.fpvy = y

    def update(self,blocks):
        self.blocks = blocks
        self.playerImg = self.imageList[0]
        pressed_key = pygame.key.get_pressed()

        if pressed_key[K_RIGHT]:
            self.change_speed(10, 0)
        if pressed_key[K_LEFT]:
            self.change_speed(-10, 0)
        if pressed_key[K_UP]:
            self.change_speed(0, -10)
        if pressed_key[K_DOWN]:
            self.change_speed(0, 10)

        self.x_collision()

        self.y_collision()

        # self.rectを更新するついでに整数座標に戻します。
        self.rect.x = int(self.fpx)
        self.rect.y = int(self.fpy)

        sys.stdout.flush()

    def draw(self, screen):
        screen.blit(self.playerImg, self.rect)

    def x_collision(self):
        width = self.rect.width
        height = self.rect.height

        # 移動先の矩形と座標を求めます。
        newx = self.fpx+self.fpvx
        newrect =Rect(newx,self.fpy,width,height)
        for block in self.blocks:
            collide = newrect.colliderect(block.rect)
            if collide:
                if self.fpvx > 0:
                    self.fpx = block.rect.left - width
                    self.fpvx = 0
                elif self.fpvx < 0:
                    self.fpx = block.rect.right
                    self.fpvx = 0
                break
            else:
                self.fpx = newx

    def y_collision(self):
        width = self.rect.width
        height = self.rect.height

        newy = self.fpy+self.fpvy
        newrect = Rect(self.fpx,newy,width,height)
        for block in self.blocks:
            collide = newrect.colliderect(block.rect)
            if collide:
                if self.fpvy > 0:
                    self.fpy = block.rect.top - height
                    self.fpvy = 0
                elif self.fpvy < 0:
                    self.fpy = block.rect.bottom
                    self.fpvy = 0
                break
            else:
                self.fpy = newy

# 画像を読み込む
def image_load(filename):
    try:
        image = pygame.image.load(filename)
    except pygame.error:
        print("Cannot load image:" % filename)
        raise SystemExit
    image = image.convert()
    return image


def split_image_load(image, frame_x, frame_y):
    # 画像のサイズ判定。
    size_x = image.get_width()
    size_y = image.get_height()

    imageList = []

    for j in range(0, size_y, frame_y):
        for i in range(0, size_x, frame_x):
            surface = pygame.Surface((frame_x, frame_y))
            surface.blit(image, (0, 0), (i, j, frame_x, frame_y))
            surface.set_colorkey(surface.get_at((0, 0)), RLEACCEL)
            surface.convert(surface)
            imageList.append(surface)

    return imageList


class Map():
    def __init__(self):
        self.image = image_load("PacBlock.bmp")
        self.imageList = split_image_load(self.image, 32, 32)

    def draw(self, screen,blocks):
        transColor = self.image.get_at((0, 0))
        self.image.set_colorkey(transColor, RLEACCEL)
        for y in range(MAP_HEIGHT):
            for x in range(MAP_WIDTH):
                if MAP[y][x] == BLOCK:
                    cell = Block(self.imageList[1], (x * CHIPSIZE, y * CHIPSIZE), screen)
                    blocks.add(cell)
                elif MAP[y][x] == FOOD:
                    screen.blit(self.imageList[2], (x * CHIPSIZE, y * CHIPSIZE))
                else:
                    pass


class Block(pygame.sprite.Sprite):
    def __init__(self,image,pos,screen):
        self.image = image
        pygame.sprite.Sprite.__init__(self)
        self.rect = self.image.get_rect()
        self.rect.topleft = pos
        screen.blit(self.image,pos)

# メイン関数。
def Main():
    pygame.init()
    screen = pygame.display.set_mode((640, 480))
    all = pygame.sprite.RenderUpdates()
    blocks = pygame.sprite.Group()
    pacman = Player()
    maze = Map()
    all.add(pacman)
    clock = pygame.time.Clock()
    # メインループ
    while True:
        clock.tick(60)

        screen.fill((0, 0, 0))

        maze.draw(screen,blocks)

        all.update(blocks)

        all.draw(screen)

        pygame.display.update()  # 画面を更新

        # イベント処理
        for event in pygame.event.get():
            if event.type == QUIT:  # 閉じるボタンが押されたら終了
                pygame.quit()  # Pygameの終了(画面閉じられる)
                sys.exit()


if __name__ == '__main__':
    Main()
